#!/usr/bin/perl

# decodefpga.pl
# John Jacobsen IT Services, for LBNL
# Decoding of partial list of FPGA register values as presented
# in DOR driver proc file
# July, 2003

use strict;

sub bit {
    my $reg = shift;
    my $regx = sprintf "%x", $reg;
    my $field = shift;
    my $result = ($reg & (0x01<<$field));
    # print "Bit $field reg $reg (0x$regx) result $result\n";
    return $result != 0;
}

my @lines;
while(<>) {
    push @lines, $_;
}

print @lines;
print "\n\n";
my $ctrl = (grep /CTRL/, @lines)[0];
$ctrl =~ /0x(\S+)/;
print "\nCNTRL register $1:\n";
my $ctrlval = hex($1);

if($ctrlval & 0x0F) {
    print "Power on: ";
    for(0..3) {
	print "$_ " if(bit($ctrlval, $_));
    }
    print "\n";
} else {
    print "No wire pairs have power applied.\n";
}

if($ctrlval & 0xF0) {
    print "Module reset: ";
    for(0..3) {
	print "$_ " if bit($ctrlval, $_+4);
    }
} else {
    print "Module reset: no wire pairs selected.\n";
}

print "RS485 ".(bit($ctrlval, 8)?"selected.\n":"not selected.\n");
print "Comms timeout ".(bit($ctrlval, 30)?"selected.\n":"not selected.\n");
print "Local oscillator ".(bit($ctrlval, 31)?"selected.\n":"not selected.\n");

my $gstat = (grep /GSTAT/, @lines)[0];
$gstat =~ /0x(\S+)/;
print "\nGSTAT register $1:\n";
my $gstatval = hex($1);

print "Cable status ".(bit($gstatval,2) ? "DID" : "DID NOT")." change.\n";
print "Current overload ".(bit($gstatval,3) ? "IS SET!!!\n" : "is not set.\n");
print "".(bit($gstatval,8)?"One or more TX buffers ARE":"No TX buffers are")." almost empty.\n";

if(bit($gstatval,9)) {
    print "One or more RX buffers have a message received.\n";
} else {
    print "No RX buffers have a message received.\n";
}

print "ERR_DET ".(bit($gstatval, 15)?"IS":"is not")." asserted.\n";
print "Global interrupt ".(bit($gstatval, 16)?"IS":"is not")." asserted.\n";

my $ttsic = (grep /TTSIC/, @lines)[0];
$ttsic =~ /0x(\S+)/||die"Bad TTSIC register";
print "\nTTSIC register $1:\n";
my $ttsicval = hex($1);

print "Channels with TX buffer empty: ";
if($ttsicval & 0xFF) {
    for(0..7) {
	print "$_ " if (bit($ttsicval,$_));
    }
    print "\n";
} else {
    print "NONE.\n";
}

print "Channels with TX buffer almost empty: ";

if($ttsicval & 0xFF00) {
    for(0..7) {
        print "$_ " if (bit($ttsicval,$_+8));
    }
    print "\n";
} else {
    print "NONE.\n";
}

print "Channels with TX interrupt enabled: ";
if($ttsicval & 0xFF000000) {
    for(0..7) {
        print "$_ " if (bit($ttsicval,$_+24));
    }
    print "\n";
} else {
    print "NONE.\n";
}

my $rtsic = (grep /RTSIC/, @lines)[0];
$rtsic =~ /0x(\S+)/||die"Bad RTSIC register";
print "\nRTSIC register $1:\n";
my $rtsicval = hex($1);

print "Channels with RX buffer empty: ";
if($rtsicval & 0xFF) {
    for(0..7) {
        print "$_ " if (bit($rtsicval,$_));
    }
    print "\n";
} else {
    print "NONE.\n";
}

print "Channels with RX buffer almost full: ";
if($rtsicval & 0xFF00) {
    for(0..7) {
        print "$_ " if (bit($rtsicval,$_+8));
    }
    print "\n";
} else {
    print "NONE.\n";
}

print "Channels with a message received: ";
if($rtsicval & 0xFF0000) {
    for(0..7) {
        print "$_ " if (bit($rtsicval,$_+16));
    }
    print "\n";
} else {
    print "NONE.\n";
}

print "Channels with RX interrupt enabled: ";
if($rtsicval & 0xFF000000) {
    for(0..7) {
        print "$_ " if (bit($rtsicval,$_+24));
    }
    print "\n";
} else {
    print "NONE.\n";
}


my $doms = (grep /DOMS/, @lines)[0];
$doms =~ /0x(\S+)/||die"Bad DOMS register";
print "\nDOMS register $1:\n";
my $domsval = hex($1);

print "Channels with DOM detected: ";
if($domsval & 0xFF) {
    for(0..7) {
        print "$_ " if (bit($domsval,$_));
    }
    print "\n";
} else {
    print "NONE.\n";
}

print "Channels with DOM timeout: ";
if($domsval & 0xFF00) {
    for(0..7) {
        print "$_ " if (bit($domsval,$_+8));
    }
    print "\n";
} else {
    print "NONE.\n";
}


print "Channels with RX buffer full: ";
if($domsval & 0xFF0000) {
    for(0..7) {
        print "$_ " if (bit($domsval,$_+16));
    }
    print "\n";
} else {
    print "NONE.\n";
}

print "Channels with a messaging received error: ";
if($domsval & 0xFF000000) {
    for(0..7) {
        print "$_ " if (bit($domsval,$_+24));
    }
    print "\n";
} else {
    print "NONE.\n";
}









