#!/usr/bin/perl

# fvers.pl 
# jacobsen@npxdesigns.com
# Started: March 2005
# Print out driver and firmware status information

package MY_PACKAGE;
use strict;
use Getopt::Long;

sub driver_present;
sub show_firmware_versions;
sub usage { 
    return <<EOF;
Usage: $0 [options] - show driver/FPGA versions
Options:  -h     : show help message
          -p     : show comms. parameters
EOF
;
}

my $help;
my $showparms = 0;

GetOptions("help|h"          => \$help,
	   "p"               => \$showparms) || die usage;

die usage if $help;

die "No driver present!\n" unless driver_present;
print show_firmware_versions;
exit;



sub driver_present { return -e "/proc/driver/domhub"; }

sub pfval {
    my $pf = shift; die unless defined $pf;
    die "$pf not found!\n" unless -e $pf;
    my $r = `cat $pf`; 
    if($r =~ /:\s+(\d+)/) {
	return $1;
    } else {
	return $r;
    }
}

sub show_firmware_versions {
    my $resp = "";
    my @cards = </proc/driver/domhub/card*>;
    
    $resp .= "Driver ".`cat /proc/driver/domhub/revision`; 
    
    my $fw;
    foreach my $card(@cards) {
	my $rev = `cat $card/rev`; chomp $rev;
	my @fpga = `cat $card/fpga`;
	$card =~ /card(\d+)/; my $cardnum = $1;
	for(@fpga) {
	    if(/FREV/) {
		if(/FREV\s+0x([0-9a-f][0-9a-f])0(\w)(\w\w)(\w\w)/) {
		    my $pcirev = hex($1);
		    $fw = $2.($rev==1?$3:hex($3)).chr(hex($4));
		    $resp .= "$card -- PCI=$pcirev, FW=$fw, DOR=$rev";
		    if($showparms) {
			$resp .= "\nautodac = ".pfval($card."/autodac");
			$resp .= "\tdacmax  = ".pfval($card."/dacmax");
			$resp .= "\tminclev = ".pfval($card."/minclev");
			$resp .= "\tmaxclev = ".pfval($card."/maxclev");
			$resp .= "\nrdelay  = ".pfval($card."/rdelay");
			$resp .= "\tsdelay  = ".pfval($card."/sdelay");
			$resp .= "\tthresh  = ".pfval($card."/thresh");
			$resp .= "\n";
		    }
		    $resp .= "\n";
		} else {
		    $resp .= "$card -- Corrupt FREV: $_.\n";
		}
	    }   
	}
    }
    return $resp;
}


__END__

