#!/usr/bin/perl

# set_dor_params.pl
# John Jacobsen, NPX Designs, Inc., jacobsen\@npxdesigns.com
# Started: Tue Sep 13 15:06:21 2005

package MY_PACKAGE;
use strict;

my $parmfile = "/usr/local/dor/conf/dor_params.conf";

die "$parmfile doesn't exist -- old driver version?!?\n"
    unless -f $parmfile;

my $contents = `cat $parmfile`;

sub getparm { 
    my $key = shift; die unless defined $key;
    if($contents =~ /$key\s*=\s*(\d+)/) {
	return $1;
    } else {
	die "Value for key $key not found in :\n$contents\n";
    }
}

my $defautodac = getparm("autodac");
my $defdacmax  = getparm("dacmax");
my $defminclev = getparm("minclev");
my $defmaxclev = getparm("maxclev");
my $defrdelay  = getparm("rdelay");
my $defsdelay  = getparm("sdelay");
my $defthresh  = getparm("thresh");

print "Setting parameters for all DOR cards...\n";
print "(all values in decimal; return selects [default])\n";

sub getkbd {
    my $what = shift; die unless defined $what;
    my $min  = shift; die unless defined $min;
    my $max  = shift; die unless defined $max;
    my $def  = shift; die unless defined $def;
    while(1) {
	print "Value for $what (min $min, max $max) [$def]: ";
	my $resp = <STDIN>; chomp $resp;
	if($resp =~ /^\s*(\d+)\s*$/ && $resp <= $max && $resp >= $min) {
	    return $resp;
	}
	return $def if $resp eq "";
    }
}
	 
my $autodac = getkbd("autodac", 0, 1,    $defautodac);
my $dacmax  = getkbd("dacmax",  0, 3,    $defdacmax);
my $minclev = getkbd("minclev", 0, 1023, $defminclev);
my $maxclev = getkbd("maxclev", 0, 1023, $defmaxclev);
my $rdelay  = getkbd("rdelay",  1, 255,  $defrdelay);
my $sdelay  = getkbd("sdelay",  1, 255,  $defsdelay);
my $thresh  = getkbd("thresh",  0, 255,  $defthresh);

sub load {
    my $name = shift; die unless defined $name;
    my $val  = shift; die unless defined $val;
    foreach my $card (</proc/driver/domhub/card?>) {
	open(PF, ">$card/$name") || die "Can't open $card/$name for writing: $!\n";
	print PF "$val\n";
	close PF;
    }
}

load("autodac", $autodac);
load("dacmax",  $dacmax);
load("minclev", $minclev);
load("maxclev", $maxclev);
load("rdelay",  $rdelay);
load("sdelay",  $sdelay);
load("thresh",  $thresh);

$contents =~ s/autodac\s*=\s*(\d+)/autodac = $autodac/;
$contents =~ s/dacmax\s*=\s*(\d+)/dacmax  = $dacmax/;
$contents =~ s/minclev\s*=\s*(\d+)/minclev = $minclev/;
$contents =~ s/maxclev\s*=\s*(\d+)/maxclev = $maxclev/;
$contents =~ s/rdelay\s*=\s*(\d+)/rdelay  = $rdelay/;
$contents =~ s/sdelay\s*=\s*(\d+)/sdelay  = $sdelay/;
$contents =~ s/thresh\s*=\s*(\d+)/thresh  = $thresh/;

print "Writing $parmfile... ";
open(F,">$parmfile") || 
    die "Can't open $parmfile: $!\n"
    .   "Parameters set but not won't last beyond reboot/reload "
    .   "of driver.\n";
print F $contents;
close F;

print "done.\n";
__END__

