/* Thanks to Arthur for this one, modded by JJ to have any number of queues */
#ifndef __DBGQ_H__
#define __DBGQ_H__
#define NDBGQBITS    8
#define NDBGQENTRIES (1<<NDBGQBITS)
#define DBGQMASK    (NDBGQENTRIES-1)
#define DBGSTRLEN   200

typedef struct dqTypeStruct {
  char dq[NDBGQENTRIES][DBGSTRLEN];
  unsigned dqHead, dqTail;
} dbgQueue;

static inline void emptyDbgQ(dbgQueue *q) { q->dqHead = q->dqTail = 0; }
static inline int getDbgQHead(dbgQueue *q) { return q->dqHead&DBGQMASK; }
static inline int getDbgQTail(dbgQueue *q) { return q->dqTail&DBGQMASK; }
static inline int nDbgQEntries(dbgQueue *q) { return q->dqHead - q->dqTail; }
static inline int dbgQisFull(dbgQueue *q) { return nDbgQEntries(q) == NDBGQENTRIES; }
static inline int dbgQisEmpty(dbgQueue *q) { return q->dqHead == q->dqTail; }
static inline char * getDbgQ(dbgQueue *q) { /* Return pointer only */
  char * ret = q->dq[getDbgQTail(q)];
  q->dqTail++;
  return ret;
}
#define  DEBUG_BUFFER_PACKET_DETAILS
#undef   DEBUG_BUFFER_PACKET_DETAILS
#ifdef   DEBUG_BUFFER_PACKET_DETAILS
#warning DEBUG_BUFFER_PACKET_DETAILS defined (undef for better performance)
static inline void printd(dbgQueue *q, const char * fmt, ...) {
  va_list args;
  char payload[DBGSTRLEN];
  if(dbgQisFull(q)) getDbgQ(q); /* Make room for a message */
  char * dqp = q->dq[getDbgQHead(q)];
  va_start(args, fmt);
  vsnprintf(payload, DBGSTRLEN-1, fmt, args);
  va_end(args);
  snprintf(dqp, DBGSTRLEN-1, "%lu %s", jiffies, payload);
  q->dqHead++;
}

#else
#define printd(...)
#endif


static dbgQueue * newDbgQ(void) {
  dbgQueue * q = (dbgQueue *) kmalloc(sizeof(dbgQueue), GFP_ATOMIC);
  return q;
}


#endif
