/* dh.h - Header file for DOR driver dh.c - jacobsen@npxdesigns.com 
   Started November, 2002
   $Id: dh.h,v 1.140 2007/07/12 16:36:53 jacobsen Exp $
*/

#ifndef _DH_INCLUDE_
#define _DH_INCLUDE_

#define DH_DOR_VERSION_MAJOR    0
#define DH_DOR_VERSION_MINOR    2
#define DH_DOR_VERSION_SUBMINOR 0

/** Major number is hard-coded but could be changed to dynamic,
    per Rubini, p. 45 (ed. 1) */
#define DH_MAJOR               92

#define DH_VENDOR          0x1234  /* PCI "vendor ID", gives "Technical Corp" in lspci */
#define DH_DEVICEID        0x5678  /* PCI device ID */


#define DH_PCI_REV              9  /* This should track with REV1_PCI_FW in Makefile */

static u32 dh_firmware_ok[] = {    /* These should also track respective entries in Makefile */
  0x001400|'v', /* dor_020v.rbf */
  0x000B00|'u', /* dor_011u.rbf */
  0x010400|'q'  /* com_104q.rbf */
};

#define DH_N_FIRMWARE_OK (sizeof(dh_firmware_ok)/sizeof(dh_firmware_ok[0])) 

/* NOTE: (DH_MAX_MESSAGE_SIZE+DH_MAX_SW_PKTSIZ-1)/DH_MAX_SW_PKTSIZ must be <  DH_MAX_IN_MSGS! */

#define DH_MAX_MESSAGE_SIZE 4092     /* Max size from userland POV.  Was 4096 but should be
				        4092 because Arthur only has room for 1 page including
				        header. */
/* Make this 500 bytes now; kalle says that will work with a full-bore echo test */
#define DH_MAX_HW_PKT_WORDS 150
#define DH_MAX_HW_PKTSIZ    (DH_MAX_HW_PKT_WORDS*4)
                                     /* HARDWARE packet max size; software packet
				        size is defined as DH_MAX_SW_PKTSIZ in dhswpkt.h. */
//#define DH_TX_FIFO_DEPTH    DH_MAX_HW_PKTSIZ+sizeof(u32)
#define DH_TX_FIFO_DEPTH    ((256 - 4)*4) 
                                     /* Last 3 bytes not really available -- used for
				         "pipeline synchronization" (per KHS/ALJ) */
#define DH_TX_AEF_SIZE      412      /* FIFO almost empty will occur when this many or fewer
				         bytes remain in FIFO */

#define DH_MAX_FIFO_HOLD_TIME (1000*DH_MAX_HW_PKTSIZ*HZ/45000) 
                                     /* max timeout in jiffies for dead dom unblocking */

#define DH_MAX_ACK_MSGS     100
#define DH_MAX_RETX_MSGS    100    

#define DH_MAX_FIFO_DEPTH   256      /* Used for drain function */


/** Defaults for "v-level" firmware with programmable comms settings */
#define DH_DEF_AUTODAC      1        /*Default autodac setting (1=on) */
#define DH_DEF_DACMAX       2
#define DH_DEF_MINCLEV      960
#define DH_DEF_MAXCLEV      970
#define DH_DEF_RDELAY       10
#define DH_DEF_SDELAY       255
#define DH_DEF_THRESH       80

/** Defaults for voltage / current min/max limits */
#define DH_DEF_MINCUR       20  /* mA */
#define DH_DEF_MAXCUR       140 /* mA */
#define DH_DEF_MINVOLTS     70  /* V */
#define DH_DEF_MAXVOLTS     100 /* V */

/** Module mode allowed parameters and their magic numbers */
#define DH_NORMAL_STR       "normal"
#define DH_SIM_NOCOPY_STR   "sim_nocopy"
#define DH_SIM_COPY_STR     "sim_copy"
#define DH_SIM_QUEUE_STR    "sim_queue"
#define DH_NORMAL           0
#define DH_SIM_NOCOPY       1
#define DH_SIM_COPY         2
#define DH_SIM_QUEUE        3


/* DOM Hub parameters (number of cards, channels, etc.) */
#define DH_MAX_CARDS        8 
#define DH_PAIRS_PER_CARD   4
#define DH_DOMS_PER_PAIR    2
#define DH_DOMS_PER_CARD    DH_PAIRS_PER_CARD * DH_DOMS_PER_PAIR
#define DH_DOMHUB_DOMS      DH_MAX_CARDS*DH_PAIRS_PER_CARD*DH_DOMS_PER_PAIR

#define DH_FLASH_PAGES      4          /**< Number of pages on the DOR flash chips */

#define DH_TCALIB_THRESH  900          /**< Time calibration threshold value, per Kalle */
//#define DH_COMM_THRESH      500        /**< Communications peak-to-valley threshold (mV) */
//#define DH_COMM_THRESH      150        /**< Communications peak-to-valley threshold (mV) 
//                                              <-- FOR LONG CABLE DOR-DOR LOOPBACK */
#define DH_DU_DT            96         /**< dU/dt value for communications (ask Kalle) */
//#define DH_DU_DT            48         /**< dU/dt value for communications (ask Kalle) */
#define DH_LOG_SECTOR       30         /**< absolute sector number of dor-test logfile */

/* Register definitions - see K. Sulanke API document, DOR-API Rev. 1.0 or higher */

#define DH_MAX_REGIS        32         /**< Max number of PCI/FPGA registers per card */

#define DH_RG_CTRL          0          /**< Control register */
#define DH_RG_GSTAT         1          /**< Global Status Register */
#define DH_RG_DSTAT         2          /**< Detailed Status Register */
#define DH_RG_TTSIC         3          /**< Tx Transfer Status Register */
#define DH_RG_RTSIC         4          /**< Rx Transfer Status Register */
#define DH_RG_INTEN         5          /**< Interrupt enable register */
#define DH_RG_DOMS          6          /**< DOM status register */
#define DH_RG_MRAR          7          /**< master read address register */
#define DH_RG_MRTC          8          /**< master read transfer count */
#define DH_RG_MWAR          9          /**< master write address register */
#define DH_RG_MWTC          10         /**< master write transfer count */
#define DH_RG_UTCRD0        11         /**< read out UTC value (low bits) */
#define DH_RG_UTCRD1        12         /**< read out UTC value (high bits) */
#define DH_RG_CURL          13         /**< current limits */
#define DH_RG_DCUR          14         /**< dom current */
#define DH_RG_FLASH         15         /**< Flash read/write register */
#define DH_RG_MBF0          16         /**< Rx message buffer, DOM_0 */
#define DH_RG_MBF1          17         /**< Rx message buffer, DOM_1 */
#define DH_RG_MBF2          18         /**< Rx message buffer, DOM_2 */
#define DH_RG_MBF3          19         /**< Rx message buffer, DOM_3 */
#define DH_RG_MBF4          20         /**< Rx message buffer, DOM_4 */
#define DH_RG_MBF5          21         /**< Rx message buffer, DOM_5 */
#define DH_RG_MBF6          22         /**< Rx message buffer, DOM_6 */
#define DH_RG_MBF7          23         /**< Rx message buffer, DOM_7 */
#define DH_RG_DOMC          24         /**< DOM control register */
#define DH_RG_CERR          25         /**< Communications errors */
#define DH_RG_TCSIC         26         /**< Time calibration status register */
#define DH_RG_TCBUF         27         /**< Time calibration data FIFO */
#define DH_RG_TSTRG         28         /**< Time string data avail. here */
#define DH_RG_DOMID         29         /**< DOM ID readout register */
#define DH_RG_DCREV         30         /**< DOM Communications module rev. */
#define DH_RG_FREV          31         /**< Firmware revision */

#define DH_NUM_REGIS        DH_RG_FREV+1 /**< Always make this the last register, + 1 */
#define DH_TSBYTES          22         /**< Number of bytes per time string */
#define SOH                 1          /**< For GPS time string decoding */
#define COL                 ':'        /**< "                          " */
#define CR                  13         /**< "                          " */
#define LF                  10         /**< "                          " */

static char * dh_reginame[32] = {
  "CTRL",    /* 0 */ 
  "GSTAT",   /* 1 */ 
  "DSTAT",   /* 2 */ 
  "TTSIC",   /* 3 */ 
  "RTSIC",   /* 4 */ 
  "INTEN",   /* 5 */ 
  "DOMS",    /* 6 */ 
  "MRAR",    /* 7 */ 
  "MRTC",    /* 8 */ 
  "MWAR",    /* 9 */ 
  "MWTC",    /* 10 */
  "CKCT",    /* 11 */
  "DATE",    /* 12 */
  "CURL",    /* 13 */
  "DCUR",    /* 14 */
  "FLASH",   /* 15 */
  "MBF0",    /* 16 */
  "MBF1",    /* 17 */
  "MBF2",    /* 18 */
  "MBF3",    /* 19 */
  "MBF4",    /* 20 */
  "MBF5",    /* 21 */
  "MBF6",    /* 22 */
  "MBF7",    /* 23 */
  "DOMC",    /* 24 */
  "unused!", /* 25 */
  "TCSIC",   /* 27 */
  "unused!", /* 28 */
  "DOMID",   /* 29 */
  "DCREV",   /* 30 */
  "FREV",    /* 31 */
};

#define DH_BITN(N) (0x1 << (N))        /**< Nice trick for selecting bit N via mask */


/* Per-register bit masks ********************************************************************/

/* Bit masks for register CTRL */
//#define DH_CTRL_TX_CHAN_RES       DH_BITN(3)  /**< Use with LOCAL_OSC_EN and bits 1&2 (wire pair select) */
//#define DH_CTRL_RX_CHAN_RES       DH_BITN(4)  /**< "" */
//#define DH_CTRL_TEST_MEM_ADDR_INC DH_BITN(16) /**< When high, each mem access increments 
//						   memory address (TEST API ONLY) */
//#define DH_CTRL_DOR_COM_RES DH_BITN(4)  /**< communications reset bit (toggle to init comms) */
#define DH_CTRL_CLR_COM_RES   DH_BITN(20) /**< For clearing stale communications reset */
#define DH_CTRL_GLOBAL_RES    DH_BITN(23) /**< Global comms reset (Rev 1) */
#define DH_CTRL_FPGA_RELOAD   DH_BITN(26) /**< Mask to reload FPGA from FLASH */
#define DH_CTRL_MR_RD_EN      DH_BITN(27) /**< DMA read enable (to DOR buf from driver) */
#define DH_CTRL_MR_WR_EN      DH_BITN(28) /**< DMA write enable (to driver from DOR buf) */
#define DH_CTRL_MEM_RD_MULT   DH_BITN(29) /**< Assert for faster bus-master writes (DMA reads) */
#define DH_CTRL_COM_TOUT_EN   DH_BITN(30) /**< Timeout enabled if DOM doesn't respond 
					     (c.f. DOMS reg) */
#define DH_CTRL_LOCAL_OSC_EN  DH_BITN(31) /**< Used to be for comms reset, now used to select
					       local vs. global clock source */

/* Bit masks for GSTAT */
#define DH_GSTAT_WP_PON_RDY   DH_BITN(1)  /**< Poll on this bit for power on */
#define DH_GSTAT_WP_POFF_RDY  DH_BITN(2)  /**< Poll on this bit for power off */
#define DH_GSTAT_WP_POW_FAIL  DH_BITN(3)  /**< Card-wise power on failure bit */
#define DH_GSTAT_UTC_LD_ENA   DH_BITN(4)  /**< Set to load UTC at next 1pps */
#define DH_GSTAT_UTC_LD_SEL   DH_BITN(5)  /**< Sets low or high order bits for UTCLD */
#define DH_GSTAT_MR_WR_TC     DH_BITN(6)  /**< Poll to see if DMA write finished */
#define DH_GSTAT_MR_RD_TC     DH_BITN(7)  /**< Poll to see if DMA read finished */
#define DH_GSTAT_PPS_TEST     DH_BITN(11) /**< DOR timer bit, pull high to load UTC @ next 1pps */
#define DH_GSTAT_GLOBAL_INT   DH_BITN(16) /**< OR of all interrupt lines on the card */
#define DH_GSTAT_MBFn_AEF     DH_BITN(8)  /**< Indicates one or more TX FIFO has room */
#define DH_GSTAT_MBFn_MRCVD   DH_BITN(9)  /**< Indicates one or more RX FIFO has message */
#define DH_GSTAT_TSTRG_10     DH_BITN(12) /**< Indicates 10 time strings available */
#define DH_GSTAT_TSTRG_1      DH_BITN(13) /**< Indicates 1 time string available */
#define DH_GSTAT_TSTRG_F_FF   DH_BITN(14) /**< Indicates 11 time strings avail., FIFO full */

/* Bit masks for TTSIC/RTSIC */
#define DH_TTSIC_TX_AEF_0__7      0x0000FF00  /**< Mask to see if any channel has room in its
						   TX FIFO */
#define DH_TTSIC_INT_EN_0__7      0xFF000000  /**< Per-channel TX interrupt masks */
#define DH_RTSIC_INT_EN_0__7      0xFF000000  /**< Per-channel RX interrupt masks */
#define DH_RTSIC_RX_MRCVD_0__7    0x00FF0000  /**< Mask to see if any channel has a message in
					    	   RX FIFO */
/* Bit masks for FLASH register */
#define DH_FL_WE                  DH_BITN(20) 
#define DH_FLASH_FL_RESET         DH_BITN(22) /**< Reset of flash state machine (does not erase) */
#define DH_FLASH_FL_RESET_VID     DH_BITN(23) /**< Used for protect/unprotect */
#define DH_FLASH_RDY              DH_BITN(31) /**< Ready bit for erase, etc. */

/* Bit masks for FLASH protect/unprotect */
#define DH_FL_PROT_ADR            0x04L
#define DH_FL_UNPROT_ADR          0x84L
#define DH_FL_SECTOR_PROT         0x60L << 24
#define DH_FL_SECTOR_UNPROT       0x60L << 24
#define DH_FL_SECTOR_VER          0x40L << 24
#define DH_FL_PROT_DONE           0x01L << 24
#define DH_FL_UNPROT_DONE         0x00L
#define DH_FL_RESET_CMD           0xF0L << 24

#define DH_LOGSECTOR              30

/* Bit masks for interrupt enable register INTEN */
#define DH_INTEN_ON_MSG_SNDBL     DH_BITN(8) /**< Enable/disable interrupt on message sendable (Tx FIFO not full) */
#define DH_INTEN_ON_MSG_RCVD      DH_BITN(9) /**< Enable/disable interrupt on message received (Rx FIFO not empty) */
#define DH_INTEN_GLOBAL_INT_EN    DH_BITN(16) /**< Global interrupt enable/disable */

/* Bit fields for CERR */
#define DH_CERR_COMM_ERR_CLR      DH_BITN(31)

/** Calculate "sister" device number for simulation (connect 0<->64, 1<->65, ...) */
#define DH_SISTER(minor) \
     (minor >= DH_DOMHUB_DOMS/2 ? minor - DH_DOMHUB_DOMS/2 : minor + DH_DOMHUB_DOMS/2)

/** Command codes (for decoding Kalle's communication protocol bytes during diagnostic dump) */
#define COM_STF     1
#define COM_EOF     2
#define COM_IDREQ   3
#define COM_DRREQ   5
#define COM_DRAND   6
#define COM_DRBT    7
#define COM_MRWB    8
#define COM_MRNB    9
#define COM_MRWE   10
#define COM_COMRES 11
#define COM_BFSTAT 12
#define COM_SYSRES 13
#define COM_TCAL   14
#define COM_IDLE   15

#define DH_MAX_DEBUG_FIFO_READS 256

static char * dh_comprot[] = {
  "unused!", /* 0 */
  "STF", /* 1 */
  "EOF", /* 2 */
  "IDREQ", /* 3 */
  "unused!", /* 4 */
  "DRREQ", /* 5 */
  "DRAND", /* 6 */
  "DRBT", /* 7 */
  "MRWB", /* 8 */
  "MRNB", /* 9 */
  "MRWE", /* 10 */
  "COMRES", /* 11 */
  "BFSTAT", /* 12 */
  "SYSRES", /* 13 */
  "TCAL", /* 14 */
  "IDLE" /* 15 */
};

/** Driver error codes (for lasterr proc file).
    dh_errs must be set up in accordance with the list of
    #define'd integer codes. */

#define DH_ERR_NONE          0
#define DH_ERR_FPGA_COMMPROG 1
#define DH_ERR_FLASHTIMEOUT  2
#define DH_ERR_BADFLASHSECT  3
#define DH_ERR_FLASHNOTEMPTY 4
#define DH_ERR_TXFIFOFULL    5
#define DH_ERR_INVALIDARG    6
#define DH_ERR_COMMSNOTRESET 7
#define DH_ERR_DOMNOTDETECT  8
#define DH_ERR_PWR2UNPLUGGED 9
#define DH_ERR_QCREATFAIL    10
#define DH_ERR_DOMNOTAVAIL   11
#define DH_ERR_BADMINOR      12
#define DH_ERR_BADUSERBUFSIZ 13
#define DH_ERR_QREADFAIL     14
#define DH_ERR_QWRITEFAIL    15
#define DH_ERR_QSHORTWRITE   16
#define DH_ERR_QCANTMOVMSG   17
#define DH_ERR_CORRUPTMSG    18
#define DH_ERR_DMASETUPFAIL  19
#define DH_ERR_DMAFAIL       20
#define DH_ERR_CANTGETOUTMSG 21
#define DH_ERR_UNHDLEDINT    22
#define DH_ERR_DMANOTSUPP    23
#define DH_ERR_PCIENABFAIL   24
#define DH_ERR_PCIALLOCFAIL  25
#define DH_ERR_MEMIONOTSUPP  26
#define DH_ERR_BADCARDID     27
#define DH_ERR_BADUSERBUFPTR 28
#define DH_ERR_BADPROCCMD    29
#define DH_ERR_BADPROCARG    30
#define DH_ERR_FPGARELODFAIL 31
#define DH_ERR_BUFALLOCFAIL  32
#define DH_ERR_MEMTESTFAIL   33
#define DH_ERR_ALREADYOPEN   34
#define DH_ERR_NODORPCIINFO  35
#define DH_ERR_CANTGETIRQ    36
#define DH_ERR_BADFLASHCMD   37
#define DH_ERR_CANTGETDOMID  38
#define DH_ERR_TCALCMDINVAL  39
#define DH_ERR_TCALCHNINVAL  40
#define DH_ERR_TCALTIMEOUT   41
#define DH_ERR_TCALMISSORBAD 42
#define DH_ERR_SOFTBOOTFAIL  43
#define DH_ERR_SOFTBNOTPWRD  44
#define DH_ERR_SOFTBNOTPLGGD 45
#define DH_ERR_SOFTBNOTCOMM  46
#define DH_ERR_TCALINTR      47
#define DH_ERR_HARDTIMEOUT   48
#define DH_ERR_CANTSAVPCI    49
#define DH_ERR_CANTRESTRPCI  50
#define DH_ERR_WPPWRONTOUT   51
#define DH_ERR_RESETINVAL    52
#define DH_ERR_UNSUPPFWARE   53
#define DH_ERR_1PPSINITFAIL  54
#define DH_ERR_NOTIMERINIT   55
#define DH_ERR_TIMERTOOLATE  56
#define DH_ERR_NOTIMERRESET  57
#define DH_ERR_TIMERNOTLOAD  58
#define DH_ERR_RESEQINTR     59
#define DH_ERR_OPENNOTPWR    60
#define DH_ERR_CRESETINTR    61
#define DH_ERR_DOMSTNOPWR    62
#define DH_ERR_TOOMANYPKTS   63
#define DH_ERR_MSGTOOBIG     64
#define DH_ERR_TCALBADBUF    65
#define DH_ERR_OCRESETTOUT   66
#define DH_ERR_OCONNTOUT     67
#define DH_ERR_ALLNOTOFF     68
#define DH_ERR_CURRDTOUT     69
#define DH_ERR_SEMAINT       70

static char *dh_errs[] = { 
  "no error occurred since last query", /* 0 */
  "FPGA reload attempted while communications in progress", /* 1 */
  "flash write failed due to timeout", /* 2 */
  "flash erase failed due to bad sector number", /* 3 */
  "flash erasure failed / not empty", /* 4 */
  "write to TX FIFO failed because FIFO was full", /* 5 */
  "invalid argument passed to driver function (see kernel log for details)", /* 6 */
  "communications reset failed (reset bit never went low)", /* 7 */
  "DOM not detected by communications firmware", /* 8 */
  "attempt to power an unplugged channel", /* 9 */
  "memory allocation for queue failed", /* 10 */
  "DOM not available / not ready", /* 11 */
  "bad minor number", /* 12 */
  "buffer in user space must equal the standard buffer size"
     " (see dh.h or kernel log)", /* 13 */
  "read failed from nonempty queue", /* 14 */
  "write failed to non-full queue", /* 15 */
  "short write to output queue", /* 16 */
  "can't move message from output queue to sister input queue", /* 17 */
  "message byte count corrupted", /* 18 */
  "DMA setup failed", /* 19 */
  "DMA failed (never zeroed MWTC)", /* 20 */
  "can't get message from output queue", /* 21 */
  "unhandled interrupt -- you'd better call John or Kalle, this is a firmware problem", /* 22 */
  "DOR device claims it can't do DMA", /* 23 */
  "couldn't enable / init DOR/PCI device", /* 24 */
  "couldn't get I/O region for DOR card", /* 25 */
  "memory mapped I/O not supported", /* 26 */
  "DOR card was found with bad ID", /* 27 */
  "pointer to userspace buffer is invalid", /* 28 */
  "invalid command written to proc file (check syntax)", /* 29 */
  "invalid argument to proc file", /* 30 */
  "FPGA reload failed, you'll probably have to power-cycle, sorry... see kernel log", /* 31 */
  "couldn't get ahold of memory for a buffer", /* 32 */
  "memory test failed", /* 33 */
  "device is already open", /* 34 */
  "device has no PCI information, probably card ID is wrong", /* 35 */
  "can't get an IRQ from the kernel", /* 36 */  
  "can't perform flash operation : bad command or command arg", /* 37 */
  "can't get DOM ID from DOR firmware", /* 38 */
  "invalid time calibration control string", /* 39 */
  "attempt to perform time calibration on an unplugged or unpowered channel", /* 40 */
  "time calibration failed due to timeout", /* 41 */
  "time calibration event missing or corrupt", /* 42 */
  "soft reset of DOM failed (timeout)", /* 43 */
  "soft reset attempted on unpowered DOM", /* 44 */
  "soft reset attempted on unplugged DOM", /* 45 */
  "soft reset attempted on uncommunicating DOM", /* 46 */
  "time calibration operation interrupted", /* 47 */
  "hardware I/O timeout (on-the-wire protocol level error)", /* 48 */
  "can't save PCI configuration space", /* 49 */
  "can't restore PCI configuration space", /* 50 */
  "timeout on wire pair power on", /* 51 */
  "comms. reset attempted on unplugged or unpowered DOM", /* 52 */
  "unsupported firmware version; only FPGA load or flash programming allowed", /* 53 */
  "1pps reset bit already high, operation in progress or firmware error", /* 54 */
  "timer not initialized first", /* 55 */
  "timer load too late or not initialized", /* 56 */
  "1pps timer didn't reset after load", /* 57 */
  "1pps timer not initialized correctly", /* 58 */
  "user interrupted before DOM acknowledged protocol re-connection/resequencing", /* 59 */
  "can't power on an unpowered DOM", /* 60 */
  "user interrupted open before DOM communications reset occured", /* 61 */
  "can't get control message for DOM which is not open", /* 62 */
  "too many packets in read or write", /* 63 */
  "message too big", /* 64 */
  "incorrect time calibration buffer size (see /var/log/messages)", /* 65 */
  "communications reset timeout on open", /* 66 */
  "connection timeout on open (failed to synchronize w/ DOM)", /* 67 */
  "command given to power on all DOMs but some or all DOMs were on; "
     "please power off first", /* 68 */
  "current read operation timed out", /* 69 */
  "user opened while acquiring semaphore guarding FPGA operation", /* 70 */
};
  
#endif /* #define _DH_INCLUDE_ */


