/* New benchmarking code for DOR driver 
   Jacobsen@npxdesigns.com 
   Use rdtscll instruction (Intel only) to measure how long
   things take in system clock units 
   May 6, 2004
*/

typedef struct dh_bench_struct {
  unsigned long long t1, t2;
  unsigned long long sumdt;
  unsigned long long ndt;
  unsigned long long mindt;
  unsigned long long maxdt;
  unsigned long long lastdt;
  int started;
} dh_bench_rec_t;

#define CHECK_BENCH_START /* Only compile this in when testing on 1 DOM, else 
			     unbalanced starts & ends will fill up /var/log/messages */
#undef CHECK_BENCH_START  
#ifndef CHECK_BENCH_START
#define dh_bench_start(b) do { b.started = 1; rdtscll(b.t1); } while(0)
#else
#define dh_bench_start(b) do {                                                            \
                            if(b.started)                                                 \
			      printk(KERN_WARNING "dh: unbalanced benchmark (start)!\n"); \
                            b.started = 1;                                                \
                            rdtscll(b.t1);                                                \
			  } while(0)
#endif

inline void dh_bench_end(dh_bench_rec_t * b) {
  rdtscll(b->t2); /* Finish it */
#ifdef CHECK_BENCH_START
  if(! b->started) {
    printk(KERN_WARNING "dh: unbalanced benchmark (end)!\n");
  }
#endif
  b->started = 0;
  b->lastdt = b->t2 - b->t1;

  if(b->lastdt < b->mindt || b->mindt == 0) {
    b->mindt = b->lastdt;
  }
  if(b->lastdt > b->maxdt || b->maxdt == 0) {
    b->maxdt = b->lastdt;
  }

  b->ndt++;
  b->sumdt += b->lastdt;
}


