#ifndef _DHSWPKT_
#define _DHSWPKT_
#ifndef _DH_INCLUDE_
#error Must include dh.h first.
#endif 

/* dhswpkt.h -- header file for software (error correcting packets) */

#define FL_ACK    0x0001
#define FL_DATA   0x0000
#define FL_CI     0x0005
#define FL_IC     0x0004
#define FL_CTRL   0x0003
#define FL_SYNFIN 0x0002

#define DH_MIN_SW_PKTSIZ (0)
#define DH_MAX_SW_PKTSIZ (DH_MAX_HW_PKTSIZ-4)
#define DH_PKTS_PER_MAXIMAL_MESSAGE (DH_MAX_MESSAGE_SIZE+DH_MAX_SW_PKTSIZ-1)/DH_MAX_SW_PKTSIZ

/** Message data type */
struct dh_internal_msg_t {
  u32 nbytes;
  int synfin; /* Set to 1 to indicate end of message; else set to 0 */
  unsigned char data[DH_MAX_SW_PKTSIZ];
};

/** Generic message queue data type */
struct dh_message_queue {
  int depth;              /**< Depth of message queue */
  struct dh_internal_msg_t * msgs; 
                          /**< Allocate sizeof(dh_internal_msg_t) * depth bytes */
  int ir;                 /**< Read pointer */
  int iw;                 /**< Write pointer */
  int inr;                /**< Number of read wraps */
  int inw;                /**< Number of write wraps */
  int devno;              /**< Device (minor) number for this queue */
  int dir;                /**< Direction for this queue : 0=input; 1=output */
  int ntot;               /**< Count of all messages ever added to this queue */
  wait_queue_head_t wq;   /**< Wait queue for this message queue */
  spinlock_t lock;        /**< Exclusion lock for this queue */
};

void dhswpkt_setLen(u32 *pkt, int length) {
  const unsigned hdr = *(unsigned *) pkt;
  *(unsigned *) pkt = (hdr & 0xfffff000)|length;
}

void dhswpkt_setSeq(u32 *pkt, int seqn) {
  const unsigned hdr = *(unsigned *) pkt;
  *(unsigned *) pkt = (hdr & 0x0000ffff)|((seqn&0xffff)<<16);
}

void dhswpkt_setFlags(u32 *pkt, int flags) {
  const unsigned hdr = *(unsigned *) pkt;
  *(unsigned *) pkt = (hdr & 0xffff0fff)|((flags&0xf)<<12);
}

static inline int getType(unsigned hdr) { return (hdr&0x00007000)>>12; }

int dhswpkt_isack(u32 *pkt)  { return getType( *(unsigned *) pkt )==FL_ACK; }
int dhswpkt_isctrl(u32 *pkt) { 
  return getType( * (unsigned *) pkt )==FL_CTRL;
}

int dhswpkt_isic(u32 *pkt) { return getType(*(unsigned *) pkt)==FL_IC; }
int dhswpkt_isci(u32 *pkt) { return getType(*(unsigned *) pkt)==FL_CI; }
int dhswpkt_issynfin(u32 *pkt) { 
  return getType( *(unsigned *) pkt )==FL_SYNFIN;
}

u16 dhswpkt_flags(u32 *pkt) { return getType( *(unsigned *) pkt); }
u16 dhswpkt_seqn(u32 *pkt)  { return (*(unsigned *) pkt) >> 16; }
u16 dhswpkt_len(u32 *pkt)   { return (*(unsigned *) pkt)&0xfff; }

unsigned char * dhswpkt_data(u32 *pkt) { return (unsigned char *)pkt + 4; }

#endif
