#ifndef __RETX_H__
#define __RETX_H__

#define NRETXQBITS    5
#define NRETXQENTRIES (1<<NRETXQBITS)
#define RETXQMASK     (NRETXQENTRIES-1)
#define NRETX         NRETXQENTRIES

#define DH_MAX_RETXB_BYTES 750  /* Maximum allowed bytes in retxb, must be less than or equal to
			           NRETX * DH_MAX_MESSAGE_SIZE */

#if DH_MAX_RETXB_BYTES > (NRETX * DH_MAX_MESSAGE_SIZE)
#error DH_MAX_RETXB_BYTES must be <= NRETX * DH_MAX_MESSAGE_SIZE
#endif

/* Element of the ReTX "buffer" -- set of messages waiting for ACKs */
typedef struct retxstruct {
  unsigned len;
  unsigned short seqn;
  unsigned short nretxs;
  struct retxstruct * next;
  unsigned short pending;
  unsigned long jiffies;
  unsigned short flags;
  unsigned char data[DH_MAX_MESSAGE_SIZE];
} rtxMsg;
/* Declare as follows: static rtxMsg retxb[DH_DOMHUB_DOMS][NRETX] */


/* ReTX "queue" -- used for non-ACKed messges waiting to be re-transmitted */
typedef struct rtxTypeStruct {
  rtxMsg rtx[NRETXQENTRIES];
  unsigned rtxHead, rtxTail;
} rtxQueue;
/* Declare as follows: static rtxQueue retxq[DH_DOMHUB_DOMS] */

static inline int getRetxQHead(rtxQueue *q) { return q->rtxHead&RETXQMASK; }
static inline int getRetxQTail(rtxQueue *q) { return q->rtxTail&RETXQMASK; }

static inline void putRetxQ(rtxQueue *q, rtxMsg *m) { 
  //q->rtx[getRetxQHead(q)]=m; 
  memcpy(&(q->rtx[getRetxQHead(q)]), m, sizeof(rtxMsg));
  q->rtxHead++; 
}

static inline rtxMsg * getRetxQ(rtxQueue *q) {
  rtxMsg * ret = &(q->rtx[getRetxQTail(q)]);
  q->rtxTail++;
  return ret;
}

static inline unsigned nextRetxQEntrySize(rtxQueue *q) {
  return q->rtx[getRetxQTail(q)].len;
}

static inline int nRetxQEntries(rtxQueue *q) { return q->rtxHead - q->rtxTail; }
static inline int rtxQisFull(rtxQueue *q) { return NRETXQENTRIES == nRetxQEntries(q); }
static inline int rtxQisEmpty(rtxQueue *q) { return q->rtxHead == q->rtxTail; }
static inline void emptyRetxQ(rtxQueue *q) { q->rtxHead = q->rtxTail = 0; }

#endif
